@@ -201,18 +201,24 @@ void verbose(int lvl, const char* msg, ...)
 static int bind_port(int sock, int port, int fam)
 {
     struct sockaddr_in addr;
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(HAVE_GETADDRINFO)
     if(fam == AF_INET6) {
     	struct sockaddr_in6 addr6;
 	memset(&addr6, 0, sizeof(addr6));
 	addr6.sin6_family = AF_INET6;
     	addr6.sin6_port = (in_port_t)htons((uint16_t)port);
+#  if HAVE_DECL_IN6ADDR_ANY
 	addr6.sin6_addr = in6addr_any;
+#  else
+	memset(&addr6.sin6_addr, 0, sizeof(addr6.sin6_addr));
+#  endif
     	return bind(sock, (struct sockaddr *)&addr6, (socklen_t) sizeof(addr6));
     }
 #endif
 
+#ifndef S_SPLINT_S
     addr.sin_family = AF_INET;
+#endif
     addr.sin_port = (in_port_t)htons((uint16_t)port);
     addr.sin_addr.s_addr = INADDR_ANY;
     return bind(sock, (struct sockaddr *)&addr, (socklen_t) sizeof(addr));
@@ -331,14 +337,22 @@ handle_tcp(int tcp_sock, struct entry* entries, int *count)
 	if(tcplen >= INBUF_SIZE) {
 		log_msg("query %d bytes too large, buffer %d bytes.\n",
 			tcplen, INBUF_SIZE);
+#ifndef USE_WINSOCK
 		close(s);
+#else
+		closesocket(s);
+#endif
 		return;
 	}
 	read_n_bytes(s, inbuf, tcplen);
 
 	handle_query(inbuf, (ssize_t) tcplen, entries, count, transport_tcp, 
 		send_tcp, &userdata, do_verbose?logfile:0);
+#ifndef USE_WINSOCK
 	close(s);
+#else
+	closesocket(s);
+#endif
 
 }
 
@@ -363,11 +377,13 @@ service(void)
 	timeout.tv_sec = 0;
 	timeout.tv_usec = 0;
 	while (1) {
+#ifndef S_SPLINT_S
 		FD_ZERO(&rset);
 		FD_ZERO(&wset);
 		FD_ZERO(&eset);
 		FD_SET(udp_sock, &rset);
 		FD_SET(tcp_sock, &rset);
+#endif
 		maxfd = udp_sock;
 		if(tcp_sock > maxfd)
 			maxfd = tcp_sock;
@@ -406,7 +422,7 @@ forkit(int number)
 #endif /* USE_WINSOCK */
 #else /* HAVE_FORK */
 		pid_t pid = fork();
-		if(pid == -1) {
+		if(pid == (pid_t) -1) {
 			log_msg("error forking: %s\n", strerror(errno));
 			return;
 		}
