@@ -138,7 +138,9 @@ verify_dnssec_rrset(
 		ldns_rdf *zone_name, 
 		ldns_rdf *name,
 		ldns_dnssec_rrsets *rrset,
-		ldns_rr_list *keys) 
+		ldns_rr_list *keys,
+		time_t check_time
+		) 
 {
 	ldns_rr_list *rrset_rrs;
 	ldns_dnssec_rrs *cur_rr, *cur_sig;
@@ -158,8 +160,9 @@ verify_dnssec_rrset(
 	if (cur_sig) {
 		while (cur_sig) {
 			good_keys = ldns_rr_list_new();
-			status = ldns_verify_rrsig_keylist(rrset_rrs, 
-					cur_sig->rr, keys, good_keys);
+			status = ldns_verify_rrsig_keylist_time(
+					rrset_rrs, cur_sig->rr, 
+					keys, check_time, good_keys);
 			if (status != LDNS_STATUS_OK) {
 				if (verbosity > 0) {
 					printf("Error: %s",
@@ -464,7 +467,9 @@ verify_dnssec_name(ldns_rdf *zone_name,
 		ldns_dnssec_zone *zone,
                 ldns_rbtree_t *zone_nodes,
                 ldns_rbnode_t *cur_node,
-		ldns_rr_list *keys)
+		ldns_rr_list *keys,
+		time_t check_time
+		)
 {
 	ldns_status result = LDNS_STATUS_OK;
 	ldns_status status;
@@ -526,8 +531,9 @@ verify_dnssec_name(ldns_rdf *zone_name,
 					cur_rrset->type != LDNS_RR_TYPE_RRSIG
 				     && cur_rrset->type != LDNS_RR_TYPE_NSEC)) {
 
-				status = verify_dnssec_rrset(zone_name,
-						name->name, cur_rrset, keys);
+				status = verify_dnssec_rrset(
+						zone_name, name->name, 
+						cur_rrset, keys, check_time);
 				if (status != LDNS_STATUS_OK 
 						&& result == LDNS_STATUS_OK) {
 					result = status;
@@ -545,9 +551,14 @@ verify_dnssec_name(ldns_rdf *zone_name,
 }
 
 static ldns_status
-verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
-		ldns_rdf *zone_name, bool apexonly, int percentage) {
-
+verify_dnssec_zone(
+		ldns_dnssec_zone *dnssec_zone, 
+		ldns_rdf *zone_name, 
+		bool apexonly, 
+		int percentage,
+		time_t check_time
+		) 
+{
 	ldns_rr_list *keys;
 	ldns_rbnode_t *cur_node;
 	ldns_dnssec_rrsets *cur_key_rrset;
@@ -604,7 +615,8 @@ verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 						dnssec_zone,
 						dnssec_zone->names,
 						cur_node,
-						keys);
+						keys,
+						check_time);
 				if (status != LDNS_STATUS_OK
 						&& result == LDNS_STATUS_OK) {
 					result = status;
@@ -632,8 +644,10 @@ main(int argc, char **argv)
 	ldns_status result = LDNS_STATUS_ERR;
 	bool apexonly = false;
 	int percentage = 100;
+	struct tm tm;
+	time_t check_time = ldns_time(NULL);
 
-	while ((c = getopt(argc, argv, "ahvV:p:")) != -1) {
+	while ((c = getopt(argc, argv, "ahvV:p:t:")) != -1) {
 		switch(c) {
                 case 'a':
                         apexonly = true;
@@ -647,11 +661,15 @@ main(int argc, char **argv)
 			printf("It also checks the NSEC(3) chain, but it will "
 					"error on opted-out delegations\n");
 			printf("\nOPTIONS:\n");
-			printf("\t-a apex only, check only the zone apex\n");
-			printf("\t-p [0-100] only checks this percentage of "
-					"the zone. Defaults to 100\n");
-			printf("\t-h show this text\n");
-			printf("\t-v shows the version and exits\n");
+			printf("\t-a\t\tapex only, check only the zone apex\n");
+			printf("\t-p [0-100]\tonly checks this percentage of "
+					"the zone.\n\t\t\tDefaults to 100\n");
+			printf("\t-h\t\tshow this text\n");
+			printf("\t-t YYYYMMDDhhmmss | [+|-]offset\n\t\t\t"
+					"Set the validation time either by an "
+					"absolute time\n\t\t\tvalue or as an offset "
+					"in seconds from <now>.\n");
+			printf("\t-v\t\tshows the version and exits\n");
 			printf("\t-V [0-5]\tset verbosity level (default 3)\n");
 			printf("\nif no file is given standard input is "
 					"read\n");
@@ -674,6 +692,19 @@ main(int argc, char **argv)
                         }
                         srandom(time(NULL) ^ getpid());
                         break;
+		case 't':
+			if (strlen(optarg) == 14 && sscanf(optarg, 
+						"%4d%2d%2d%2d%2d%2d",
+						&tm.tm_year, &tm.tm_mon,
+						&tm.tm_mday, &tm.tm_hour,
+					       	&tm.tm_min, &tm.tm_sec) == 6) {
+				tm.tm_year -= 1900;
+				tm.tm_mon--;
+				check_time = mktime_from_utc(&tm);
+			}
+			else  {
+				check_time += atoi(optarg);
+			}
 		}
 	}
 
@@ -717,7 +748,7 @@ main(int argc, char **argv)
 
 		result = verify_dnssec_zone(dnssec_zone, 
 				ldns_rr_owner(ldns_zone_soa(z)),
-				apexonly, percentage);
+				apexonly, percentage, check_time);
 
 		if (result == LDNS_STATUS_OK) {
 			if (verbosity >= 1) {
