@@ -138,10 +138,11 @@ ldns_rr2buffer_wire_canonical(ldns_buffer *buffer,
 
 		for (i = 0; i < ldns_rr_rd_count(rr); i++) {
 			if (pre_rfc3597) {
-				(void) ldns_rdf2buffer_wire_canonical(buffer,
-											   ldns_rr_rdf(rr, i));
+				(void) ldns_rdf2buffer_wire_canonical(
+						buffer, ldns_rr_rdf(rr, i));
 			} else {
-				(void) ldns_rdf2buffer_wire(buffer, ldns_rr_rdf(rr, i));
+				(void) ldns_rdf2buffer_wire(
+						buffer, ldns_rr_rdf(rr, i));
 			}
 		}
 		
@@ -178,7 +179,8 @@ ldns_rr2buffer_wire(ldns_buffer *buffer, const ldns_rr *rr, int section)
 		}	
 
 		for (i = 0; i < ldns_rr_rd_count(rr); i++) {
-			(void) ldns_rdf2buffer_wire(buffer, ldns_rr_rdf(rr, i));
+			(void) ldns_rdf2buffer_wire(
+					buffer, ldns_rr_rdf(rr, i));
 		}
 		
 		if (rdl_pos != 0) {
@@ -203,9 +205,7 @@ ldns_rrsig2buffer_wire(ldns_buffer *buffer, const ldns_rr *rr)
 	/* Convert all the rdfs, except the actual signature data
 	 * rdf number 8  - the last, hence: -1 */
 	for (i = 0; i < ldns_rr_rd_count(rr) - 1; i++) {
-		if (ldns_rr_rdf(rr, i)) {
-			(void) ldns_rdf2buffer_wire(buffer, ldns_rr_rdf(rr, i));
-		}
+		(void) ldns_rdf2buffer_wire(buffer, ldns_rr_rdf(rr, i));
 	}
 
 	return ldns_buffer_status(buffer);
@@ -308,6 +308,7 @@ ldns_pkt2buffer_wire(ldns_buffer *buffer, const ldns_pkt *packet)
 	/* add EDNS to additional if it is needed */
 	if (ldns_pkt_edns(packet)) {
 		edns_rr = ldns_rr_new();
+		if(!edns_rr) return LDNS_STATUS_MEM_ERR;
 		ldns_rr_set_owner(edns_rr,
 				ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, "."));
 		ldns_rr_set_type(edns_rr, LDNS_RR_TYPE_OPT);
@@ -316,7 +317,13 @@ ldns_pkt2buffer_wire(ldns_buffer *buffer, const ldns_pkt *packet)
 		edata[1] = ldns_pkt_edns_version(packet);
 		ldns_write_uint16(&edata[2], ldns_pkt_edns_z(packet));
 		ldns_rr_set_ttl(edns_rr, ldns_read_uint32(edata));
+		/* don't forget to add the edns rdata (if any) */
+		if (packet->_edns_data)
+			ldns_rr_push_rdf (edns_rr, packet->_edns_data);
 		(void)ldns_rr2buffer_wire(buffer, edns_rr, LDNS_SECTION_ADDITIONAL);
+		/* take the edns rdata back out of the rr before we free rr */
+		if (packet->_edns_data)
+			(void)ldns_rr_pop_rdf (edns_rr);
 		ldns_rr_free(edns_rr);
 	}
 	
@@ -337,17 +344,23 @@ ldns_rdf2wire(uint8_t **dest, const ldns_rdf *rdf, size_t *result_size)
 	ldns_status status;
 	*result_size = 0;
 	*dest = NULL;
+	if(!buffer) return LDNS_STATUS_MEM_ERR;
 	
 	status = ldns_rdf2buffer_wire(buffer, rdf);
 	if (status == LDNS_STATUS_OK) {
 		*result_size =  ldns_buffer_position(buffer);
 		result = (uint8_t *) ldns_buffer_export(buffer);
 	} else {
+		ldns_buffer_free(buffer);
 		return status;
 	}
 	
 	if (result) {
 		*dest = LDNS_XMALLOC(uint8_t, ldns_buffer_position(buffer));
+		if(!*dest) {
+			ldns_buffer_free(buffer);
+			return LDNS_STATUS_MEM_ERR;
+		}
 		memcpy(*dest, result, ldns_buffer_position(buffer));
 	}
 	
@@ -363,17 +376,23 @@ ldns_rr2wire(uint8_t **dest, const ldns_rr *rr, int section, size_t *result_size
 	ldns_status status;
 	*result_size = 0;
 	*dest = NULL;
+	if(!buffer) return LDNS_STATUS_MEM_ERR;
 	
 	status = ldns_rr2buffer_wire(buffer, rr, section);
 	if (status == LDNS_STATUS_OK) {
 		*result_size =  ldns_buffer_position(buffer);
 		result = (uint8_t *) ldns_buffer_export(buffer);
 	} else {
+		ldns_buffer_free(buffer);
 		return status;
 	}
 	
 	if (result) {
 		*dest = LDNS_XMALLOC(uint8_t, ldns_buffer_position(buffer));
+		if(!*dest) {
+			ldns_buffer_free(buffer);
+			return LDNS_STATUS_MEM_ERR;
+		}
 		memcpy(*dest, result, ldns_buffer_position(buffer));
 	}
 	
@@ -389,17 +408,23 @@ ldns_pkt2wire(uint8_t **dest, const ldns_pkt *packet, size_t *result_size)
 	ldns_status status;
 	*result_size = 0;
 	*dest = NULL;
+	if(!buffer) return LDNS_STATUS_MEM_ERR;
 	
 	status = ldns_pkt2buffer_wire(buffer, packet);
 	if (status == LDNS_STATUS_OK) {
 		*result_size =  ldns_buffer_position(buffer);
 		result = (uint8_t *) ldns_buffer_export(buffer);
 	} else {
+		ldns_buffer_free(buffer);
 		return status;
 	}
 	
 	if (result) {
 		*dest = LDNS_XMALLOC(uint8_t, ldns_buffer_position(buffer));
+		if(!*dest) {
+			ldns_buffer_free(buffer);
+			return LDNS_STATUS_MEM_ERR;
+		}
 		memcpy(*dest, result, ldns_buffer_position(buffer));
 	}
 	
